package java.time;

import java.time.format.DateTimeFormatterBuilder;
import java.io.DataInput;
import java.io.IOException;
import java.io.DataOutput;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.time.temporal.Temporal;
import java.time.temporal.TemporalQueries;
import java.time.temporal.TemporalQuery;
import java.time.temporal.UnsupportedTemporalTypeException;
import java.time.temporal.ValueRange;
import java.time.temporal.TemporalField;
import java.time.chrono.Chronology;
import java.time.chrono.IsoChronology;
import java.time.temporal.ChronoField;
import java.util.Objects;
import java.time.format.DateTimeFormatter;
import java.io.Serializable;
import java.time.temporal.TemporalAdjuster;
import java.time.temporal.TemporalAccessor;

public final class MonthDay implements TemporalAccessor, TemporalAdjuster, Comparable<MonthDay>, Serializable
{
    private static final long serialVersionUID = -939150713474957432L;
    private static final DateTimeFormatter PARSER;
    private final int month;
    private final int day;
    
    public static MonthDay now() {
        return now(Clock.systemDefaultZone());
    }
    
    public static MonthDay now(ZoneId zoneId) {
        return now(Clock.system(zoneId));
    }
    
    public static MonthDay now(Clock clock) {
        LocalDate now = LocalDate.now(clock);
        return of(now.getMonth(), now.getDayOfMonth());
    }
    
    public static MonthDay of(Month obj, int i) {
        Objects.requireNonNull(obj, "month");
        ChronoField.DAY_OF_MONTH.checkValidValue((long)i);
        if (i > obj.maxLength()) {
            throw new DateTimeException("Illegal value for DayOfMonth field, value " + i + " is not valid for month " + obj.name());
        }
        return new MonthDay(obj.getValue(), i);
    }
    
    public static MonthDay of(int n, int n2) {
        return of(Month.of(n), n2);
    }
    
    public static MonthDay from(TemporalAccessor from) {
        if ((from instanceof MonthDay)) {
            return (MonthDay)from;
        }
        try {
            if (!IsoChronology.INSTANCE.equals(Chronology.from(from))) {
                from = LocalDate.from(from);
            }
            return of(from.get(ChronoField.MONTH_OF_YEAR), from.get(ChronoField.DAY_OF_MONTH));
        }
        catch (DateTimeException ex) {
            throw new DateTimeException("Unable to obtain MonthDay from TemporalAccessor: " + from + " of type " + from.getClass().getName(), ex);
        }
    }
    
    public static MonthDay parse(CharSequence charSequence) {
        return parse(charSequence, MonthDay.PARSER);
    }
    
    public static MonthDay parse(CharSequence text, DateTimeFormatter obj) {
        Objects.requireNonNull(obj, "formatter");
        return (MonthDay)obj.parse(text, MonthDay::from);
    }
    
    private MonthDay(int month, int day) {
        this.month = month;
        this.day = day;
    }
    
    @Override
    public boolean isSupported(TemporalField temporalField) {
        if ((temporalField instanceof ChronoField)) {
            return temporalField == ChronoField.MONTH_OF_YEAR || temporalField == ChronoField.DAY_OF_MONTH;
        }
        return temporalField != null && temporalField.isSupportedBy(this);
    }
    
    @Override
    public ValueRange range(TemporalField field) {
        if (field == ChronoField.MONTH_OF_YEAR) {
            return field.range();
        }
        if (field == ChronoField.DAY_OF_MONTH) {
            return ValueRange.of(1L, (long)this.getMonth().minLength(), (long)this.getMonth().maxLength());
        }
        return super.range(field);
    }
    
    @Override
    public int get(TemporalField field) {
        return this.range(field).checkValidIntValue(this.getLong(field), field);
    }
    
    @Override
    public long getLong(TemporalField obj) {
        if ((obj instanceof ChronoField)) {
            switch ((ChronoField)obj) {
                case DAY_OF_MONTH: {
                    return (long)this.day;
                }
                case MONTH_OF_YEAR: {
                    return (long)this.month;
                }
                default: {
                    throw new UnsupportedTemporalTypeException("Unsupported field: " + obj);
                }
            }
        }
        else {
            return obj.getFrom(this);
        }
    }
    
    public int getMonthValue() {
        return this.month;
    }
    
    public Month getMonth() {
        return Month.of(this.month);
    }
    
    public int getDayOfMonth() {
        return this.day;
    }
    
    public boolean isValidYear(int n) {
        return this.day != 29 || this.month != 2 || Year.isLeap((long)n);
    }
    
    public MonthDay withMonth(int n) {
        return this.with(Month.of(n));
    }
    
    public MonthDay with(Month obj) {
        Objects.requireNonNull(obj, "month");
        if (obj.getValue() == this.month) {
            return this;
        }
        return new MonthDay(obj.getValue(), Math.min(this.day, obj.maxLength()));
    }
    
    public MonthDay withDayOfMonth(int n) {
        if (n == this.day) {
            return this;
        }
        return of(this.month, n);
    }
    
    @Override
    public <R> R query(TemporalQuery<R> query) {
        if (query == TemporalQueries.chronology()) {
            return IsoChronology.INSTANCE;
        }
        return super.query(query);
    }
    
    @Override
    public Temporal adjustInto(Temporal with) {
        if (!Chronology.from(with).equals(IsoChronology.INSTANCE)) {
            throw new DateTimeException("Adjustment only supported on ISO date-time");
        }
        with = with.with(ChronoField.MONTH_OF_YEAR, (long)this.month);
        return with.with(ChronoField.DAY_OF_MONTH, Math.min(with.range(ChronoField.DAY_OF_MONTH).getMaximum(), (long)this.day));
    }
    
    public String format(DateTimeFormatter obj) {
        Objects.requireNonNull(obj, "formatter");
        return obj.format(this);
    }
    
    public LocalDate atYear(int n) {
        return LocalDate.of(n, this.month, this.isValidYear(n) ? this.day : 28);
    }
    
    @Override
    public int compareTo(MonthDay monthDay) {
        int n = this.month - monthDay.month;
        if (n == 0) {
            n = this.day - monthDay.day;
        }
        return n;
    }
    
    public boolean isAfter(MonthDay monthDay) {
        return this.compareTo(monthDay) > 0;
    }
    
    public boolean isBefore(MonthDay monthDay) {
        return this.compareTo(monthDay) < 0;
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if ((o instanceof MonthDay)) {
            MonthDay monthDay = (MonthDay)o;
            return this.month == monthDay.month && this.day == monthDay.day;
        }
        return false;
    }
    
    @Override
    public int hashCode() {
        return (this.month << 6) + this.day;
    }
    
    @Override
    public String toString() {
        return new StringBuilder(10).append("--").append((this.month < 10) ? "0" : "").append(this.month).append((this.day < 10) ? "-0" : "-").append(this.day).toString();
    }
    
    private Object writeReplace() {
        return new Ser((byte)13, this);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws InvalidObjectException {
        throw new InvalidObjectException("Deserialization via serialization delegate");
    }
    
    void writeExternal(DataOutput dataOutput) throws IOException {
        dataOutput.writeByte(this.month);
        dataOutput.writeByte(this.day);
    }
    
    static MonthDay readExternal(DataInput dataInput) throws IOException {
        return of((int)dataInput.readByte(), (int)dataInput.readByte());
    }
    
    static {
        PARSER = new DateTimeFormatterBuilder().appendLiteral("--").appendValue(ChronoField.MONTH_OF_YEAR, 2).appendLiteral('-').appendValue(ChronoField.DAY_OF_MONTH, 2).toFormatter();
    }
}
